'==========================================='
' Debug Client v.1.0
' by B. Aslan
'==========================================='
#COMPILE DLL "DebugClient.dll"

#IF NOT %DEF(%WINAPI)
    #INCLUDE "WIN32API.INC"
#ENDIF

%DEBUGCLIENT = %TRUE
#INCLUDE "DebugConst.INC"

GLOBAL DebugServer    AS LONG,  _
       DebugWindow    AS LONG,  _
       lResult        AS LONG

GLOBAL T_Win_Title    AS STRING, _
       T_Win_Handle   AS LONG


DECLARE FUNCTION IsDebugServer () AS LONG

'------------------------------------------------------------------------
' CONSTANT: Debug Window Update Method
' Debug Server contains an IPT (Inter Process Text control) control
' to receive messages from Debug Client application.
' You can use IPT control for sending/receiving an interprocess messages
' between two applications.
'
'   %DEBUG_WINDOW_UPDATE_METHOD
'      0 - Update Debug Window directly (WinAPI textbox messages)
'      1 - Update Debug Window via IPT
'------------------------------------------------------------------------
%DEBUG_WINDOW_UPDATE_METHOD = 1

'------------------------------------------------------------------------
' MACRO: SendDebugCommand
' Send Command to Debug Server application
'------------------------------------------------------------------------
MACRO SendDebugCommand ( wParam, lParam ) = DIALOG SEND DebugServer, %WM_USER + wParam, lParam, 0 TO lResult

FUNCTION LIBMAIN(BYVAL hInstance AS DWORD, _
                 BYVAL Reason    AS LONG,  _
                 BYVAL Reserved  AS LONG) AS LONG

  SELECT CASE Reason
    CASE %DLL_PROCESS_ATTACH

      IF ISTRUE IsDebugServer () THEN
         LIBMAIN     = 1
         EXIT FUNCTION
      ELSE
         MSGBOX $DEBUG_SERVER_CAPTION & " not found!", _
                %MB_OK + %MB_ICONQUESTION + %MB_SYSTEMMODAL, _
                $DEBUG_CLIENT_CAPTION

         DebugServer = 0
         LIBMAIN     = 0
         EXIT FUNCTION
      END IF

    CASE %DLL_PROCESS_DETACH
      EXIT FUNCTION

    CASE %DLL_THREAD_ATTACH
      EXIT FUNCTION

    CASE %DLL_THREAD_DETACH
      EXIT FUNCTION

  END SELECT

END FUNCTION

'------------------------------------------------------------------------
' DEBUG Subroutine.
'   Input parameter: sText - string to output
'------------------------------------------------------------------------
SUB DEBUG ALIAS "DEBUG" ( BYVAL sText AS STRING ) EXPORT
      '
      ' Check the presence of Debug Server in memory
      '
      SendDebugCommand ( %DSRV_SUPP, %DSRV_HANDSHAKE )  ' send an handshake command to debug server

      IF DebugServer > 0 AND lResult = %DSRV_HANDSHAKE THEN

      #IF %DEBUG_WINDOW_UPDATE_METHOD = 0
         '
         ' Update Debug Window directly
         '
         LOCAL sTemp AS STRING, lLength AS LONG

         lLength = SendMessage ( DebugWindow, %WM_GETTEXTLENGTH, 0, 0 )  ' get length of text inside of Debug Window
         INCR lLength
         SendMessage ( DebugWindow, %WM_GETTEXT, lLength, STRPTR(sText) )' get Debug Window text
         sTemp   = sTemp & sText & $CRLF                                 ' prepare new debug string to update
         SendMessage ( DebugWindow, %WM_SETTEXT, 0, STRPTR(sTemp) )      ' update Debug Window

      #ELSEIF %DEBUG_WINDOW_UPDATE_METHOD = 1
         '
         ' Update Debug Window via Debug Server IPT (Inter Process Text control)
         '
         SendDebugCommand ( %DSRV_HIPT, 0 )                              ' request to access to IPT hWnd
         SendMessage      ( lResult, %WM_SETTEXT, 0, STRPTR (sText) )    ' update IPT control
         SendDebugCommand ( %DSRV_SUPP, %DSRV_ADDTEXT )                  ' tell Debug Server to update Debug Window

      #ENDIF

      ELSE
         '
         ' Debug Server not found.
         '
         MSGBOX $DEBUG_SERVER_CAPTION & " not initialized!", %MB_OK + %MB_ICONERROR + %MB_SYSTEMMODAL, $DEBUG_CLIENT_CAPTION
      END IF

END SUB

$DEBUGV_PARAM_DELIM = CHR$(32)

SUB DebugV ALIAS "DEBUGV" ( OPT BYVAL v1 AS VARIANT, _
                            OPT BYVAL v2 AS VARIANT, _
                            OPT BYVAL v3 AS VARIANT, _
                            OPT BYVAL v4 AS VARIANT, _
                            OPT BYVAL v5 AS VARIANT, _
                            OPT BYVAL v6 AS VARIANT, _
                            OPT BYVAL v7 AS VARIANT, _
                            OPT BYVAL v8 AS VARIANT ) EXPORT

    DIM v(1:8) AS VARIANT, i AS LONG, n AS LONG, T AS STRING
    v(1)=v1:v(2)=v2:v(3)=v3:v(4)=v4:v(5)=v5:v(6)=v6:v(7)=v7:v(8)=v8
    FOR i = 1 TO 8
        n = VARIANTVT(v(i))
        IF n = 8 THEN
           T = T & VARIANT$(v(i)) & $DEBUGV_PARAM_DELIM
        ELSEIF n <> 0 THEN
           T = T & STR$(VARIANT#(v(i))) & $DEBUGV_PARAM_DELIM
        END IF
    NEXT i
    DEBUG T

END SUB

SUB DebugSetCmd ALIAS "DEBUGSETCMD" ( BYVAL Cmd AS LONG, OPT BYVAL CmdText AS STRING) EXPORT
    SELECT CASE Cmd

        CASE %DSRV_CLEAR
             SendDebugCommand ( %DSRV_SUPP, %DSRV_CLEAR )

        CASE %DSRV_TOPMOST
             SendDebugCommand ( %DSRV_SUPP, %DSRV_TOPMOST )

        CASE %DSRV_NONTOPMOST
             SendDebugCommand ( %DSRV_SUPP, %DSRV_NONTOPMOST )

        CASE %DSRV_COPYTOCLIPBOARD
             SendDebugCommand ( %DSRV_SUPP, %DSRV_COPYTOCLIPBOARD )

        CASE %DSRV_LOCKWINDOW
             SendDebugCommand ( %DSRV_SUPP, %DSRV_LOCKWINDOW )

        CASE %DSRV_UNLOCKWINDOW
             SendDebugCommand ( %DSRV_SUPP, %DSRV_UNLOCKWINDOW )

        CASE %DSRV_SAVETOFILE
             IF LEN(CmdText) > 0 THEN
                SendDebugCommand ( %DSRV_HIPT, 0 )                             ' request to access to IPT hWnd
                SendMessage      ( lResult, %WM_SETTEXT, 0, STRPTR (CmdText) ) ' copy filename into an IPT buffer
                SendDebugCommand ( %DSRV_SUPP, %DSRV_SAVETOFILE )              ' call 'Save to file' procedure
             END IF

        CASE %DSRV_ENABLESCROLL
             SendDebugCommand ( %DSRV_SUPP, %DSRV_ENABLESCROLL )

        CASE %DSRV_DISABLESCROLL
             SendDebugCommand ( %DSRV_SUPP, %DSRV_DISABLESCROLL )

        CASE %DSRV_SHOWTIME
             SendDebugCommand ( %DSRV_SUPP, %DSRV_SHOWTIME )

        CASE %DSRV_HIDETIME
             SendDebugCommand ( %DSRV_SUPP, %DSRV_HIDETIME )

        CASE %DSRV_SETFONT
             SendDebugCommand ( %DSRV_HIPT, 0 )                                ' request to access to IPT hWnd
             SendMessage      ( lResult, %WM_SETTEXT, 0, STRPTR (CmdText) )    ' copy font info into an IPT buffer
             SendDebugCommand ( %DSRV_SUPP, %DSRV_SETFONT )                    ' call set font procedure

        CASE %DSRV_RESTORE
             SendDebugCommand ( %DSRV_SUPP, %DSRV_RESTORE )                    ' call restore window

        CASE %DSRV_MOVEWINDOW
             SendDebugCommand ( %DSRV_HIPT, 0 )                                ' request to access to IPT hWnd
             SendMessage      ( lResult, %WM_SETTEXT, 0, STRPTR (CmdText) )    ' copy move positions
             SendDebugCommand ( %DSRV_SUPP, %DSRV_MOVEWINDOW )                 ' call set font procedure

        CASE ELSE

    END SELECT
END SUB

'------------------------------------------------------------------------
'  IsDebugServer()
'  Check whether Debug Server present in memory or not.
'  Return %TRUE on success, otherwise %FALSE
'------------------------------------------------------------------------
FUNCTION IsDebugServer () AS LONG

      T_Win_Title    = $DEBUG_SERVER_CAPTION
      EnumWindows    CODEPTR(EnumWindowsProc), 0
      DebugServer    = T_Win_Handle

      SendDebugCommand ( %DSRV_SUPP, %DSRV_HANDSHAKE )  ' check the presence of Debug Server

      FUNCTION       = %FALSE

      IF DebugServer > 0 AND lResult = %DSRV_HANDSHAKE THEN
         SendDebugCommand ( %DSRV_HTXT, 0 )
         DebugWindow = lResult
         FUNCTION    = %TRUE
      END IF

END FUNCTION

'------------------------------------------------------------------------
' EnumWindowsProc
' Callback function to enumerate running windows
'------------------------------------------------------------------------
FUNCTION EnumWindowsProc(BYVAL lHandle AS LONG, BYVAL lNotUsed AS LONG) PRIVATE AS LONG

  DIM winTitle AS LOCAL STRING
  winTitle = STRING$( 256, 0 )

  GetWindowText lHandle, BYVAL STRPTR(winTitle), 256

  winTitle = EXTRACT$ ( winTitle, CHR$(0) )

  IF LEN( winTitle ) AND winTitle = T_Win_Title THEN
     T_Win_Handle = lHandle
     FUNCTION = 0
     EXIT FUNCTION
  END IF

  FUNCTION = 1

END FUNCTION
